@@ -11,8 +11,8 @@ use crate::inventory::ReserveResult;
 use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
-use crate::payment::handlers::create_handler;
-use crate::payment::PaymentProcessor;
+use crate::payment::handlers as payment_handlers;
+use crate::payment::{Initiator, PaymentProcessor};
 use crate::shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
 
 pub mod handlers;
@@ -116,16 +116,13 @@ impl Checkout {
     async fn enter_payment_in_progress_state<C: Context + Send>(
         &mut self,
         ctx: &mut C,
-        args: <C as PaymentProcessor>::InitiateArgs,
+        args: Option<<C as PaymentProcessor>::ProcessArgs>,
         deadline: DateTime<Utc>,
     ) -> Result<(), Error> {
-        let payment = create_handler(
-            ctx,
-            self.id.clone(),
-            self.invoice.as_ref().unwrap().initial_charge_amount.clone(),
-            args,
-        )
-        .await?;
+        let charge_amount = self.invoice.as_ref().unwrap().initial_charge_amount.clone();
+        let payment =
+            payment_handlers::create_handler(ctx, Initiator::Checkout(self), charge_amount, args)
+                .await?;
         self.payment_id = Some(payment.id.clone());
         self.state = State::PaymentInProgress(deadline);
 
@@ -172,7 +169,7 @@ impl Checkout {
     async fn ensure_payment_in_progress_state<C: Context + Send>(
         &mut self,
         ctx: &mut C,
-        args: <C as PaymentProcessor>::InitiateArgs,
+        args: Option<<C as PaymentProcessor>::ProcessArgs>,
     ) -> Result<(), Error> {
         match self.state {
             State::PaymentInProgress(_) => Ok(()),
@@ -380,7 +377,7 @@ impl Checkout {
     pub async fn initiate_payment<C: Context + Send>(
         &mut self,
         ctx: &mut C,
-        args: <C as PaymentProcessor>::InitiateArgs,
+        args: Option<<C as PaymentProcessor>::ProcessArgs>,
     ) -> Result<(), Error> {
         self.ensure_payment_in_progress_state(ctx, args).await?;
 
