@@ -1,13 +1,14 @@
 use async_trait::async_trait;
 
+use crate::checkout::CheckoutStore;
 use crate::error::Error;
-use crate::event::EventHandler;
 use crate::inventory::InventoryController;
 use crate::invoice::InvoiceCalculator;
+use crate::payment::PaymentProcessor;
+use crate::payment::PaymentStore;
 use crate::price::PriceCalculator;
 use crate::product::ProductResolver;
 use crate::shipping::ShippingCalculator;
-use crate::store::CheckoutStore;
 use crate::transaction::TransactionController;
 
 #[async_trait]
@@ -17,9 +18,10 @@ pub trait Context:
     + InventoryController
     + ShippingCalculator
     + PriceCalculator
-    + EventHandler
     + TransactionController
     + CheckoutStore
+    + PaymentProcessor
+    + PaymentStore
 {
 }
 
