@@ -7,16 +7,13 @@ use crate::error::{new_server_error, Error};
 pub struct InternalContext {
     pub pubsub_client: Arc<Client>,
     pub payment_processing_topic: Arc<Topic>,
-    pub checkout_completion_topic: Arc<Topic>,
     pub payment_processing_subscription: Arc<Subscription>,
-    pub checkout_completion_subscription: Arc<Subscription>,
 }
 
 impl InternalContext {
     pub async fn new(
         google_application_credentials: String,
         payment_processing_topic_name: String,
-        checkout_completion_topic_name: String,
     ) -> Result<Self, Error> {
         let pubsub_client = Arc::new(
             cloud_pubsub::Client::new(google_application_credentials)
@@ -27,21 +24,13 @@ impl InternalContext {
         let payment_processing_topic =
             Arc::new(pubsub_client.topic(payment_processing_topic_name.clone()));
 
-        let checkout_completion_topic =
-            Arc::new(pubsub_client.topic(checkout_completion_topic_name.clone()));
-
         let payment_processing_subscription =
             Arc::new(pubsub_client.subscribe(payment_processing_topic_name));
 
-        let checkout_completion_subscription =
-            Arc::new(pubsub_client.subscribe(checkout_completion_topic_name));
-
         Ok(Self {
             pubsub_client,
             payment_processing_topic,
-            checkout_completion_topic,
             payment_processing_subscription,
-            checkout_completion_subscription,
         })
     }
 }
