@@ -3,13 +3,15 @@ use crate::error::{new_not_found_error, Error};
 use crate::internal_context::InternalContext;
 use crate::money::Money;
 
-use super::{cancel, new, process, queue_for_processing, Payment, PaymentProcessor};
+use super::{
+    cancel, new, process, queue_for_processing, Initiator, InitiatorId, Payment, PaymentProcessor,
+};
 
-pub async fn create_handler<C: Context + Send>(
+pub async fn create_handler<'a, C: Context + Send>(
     ctx: &mut C,
-    checkout_id: String,
+    initiator: Initiator<'a>,
     charge_amount: Money,
-    args: <C as PaymentProcessor>::InitiateArgs,
+    args: Option<<C as PaymentProcessor>::ProcessArgs>,
 ) -> Result<
     Payment<
         <C as PaymentProcessor>::PrivateData,
@@ -18,7 +20,7 @@ pub async fn create_handler<C: Context + Send>(
     >,
     Error,
 > {
-    new(ctx, checkout_id, charge_amount, args).await
+    new(ctx, initiator, charge_amount, args).await
 }
 
 pub async fn step_handler<C: Context + Send>(
@@ -80,17 +82,18 @@ pub async fn process_handler<C: Context + Send>(ctx: &mut C, id: &str) -> Result
 
     let mut payment = maybe_payment.unwrap();
 
-    let maybe_checkout = ctx.get_checkout_for_update(&payment.checkout_id).await?;
-    if maybe_checkout.is_none() {
-        // this isn't supposed to happen. maybe log the error and move on
-        return Ok(());
+    match &payment.initiator_id {
+        InitiatorId::Checkout(co_id) => {
+            let maybe_co = ctx.get_checkout_for_update(co_id).await?;
+            if maybe_co.is_none() {
+                // this isn't supposed to happen. maybe log the error and move on
+                return Ok(());
+            }
+
+            let mut checkout = maybe_co.unwrap();
+            process(ctx, &mut payment, Initiator::Checkout(&mut checkout)).await
+        }
     }
-
-    let mut checkout = maybe_checkout.unwrap();
-
-    process(ctx, &mut payment, &mut checkout).await?;
-
-    Ok(())
 }
 
 pub async fn cancel_handler<C: Context + Send>(
@@ -110,16 +113,17 @@ pub async fn cancel_handler<C: Context + Send>(
     }
     let mut payment = maybe_payment.unwrap();
 
-    let maybe_checkout = ctx.get_checkout_for_update(&payment.checkout_id).await?;
-    if maybe_checkout.is_none() {
-        return Err(new_not_found_error(
-            "the associated checkout could not be found",
-        ));
+    match &payment.initiator_id {
+        InitiatorId::Checkout(co_id) => {
+            let maybe_co = ctx.get_checkout_for_update(co_id).await?;
+            if maybe_co.is_none() {
+                return Err(new_not_found_error("the checkout could not be found"));
+            }
+
+            let mut checkout = maybe_co.unwrap();
+            cancel(ctx, &mut payment, Initiator::Checkout(&mut checkout)).await?;
+        }
     }
 
-    let mut checkout = maybe_checkout.unwrap();
-
-    cancel(ctx, &mut payment, &mut checkout).await?;
-
     Ok(payment)
 }
