@@ -15,7 +15,7 @@ pub mod handlers;
 pub struct Payment<PrivateData, PublicData, ProcessArgs> {
     pub id: String,
     pub state: PaymentState<ProcessArgs>,
-    pub checkout_id: String,
+    pub initiator_id: InitiatorId,
     pub charge_amount: Money,
     pub data: PaymentData<PrivateData, PublicData>,
 }
@@ -28,25 +28,21 @@ pub enum PaymentState<ProcessArgs> {
     Cancelled,
 }
 
+#[derive(Serialize, Deserialize, JsonSchema)]
+pub enum InitiatorId {
+    Checkout(String),
+}
+
+pub enum Initiator<'a> {
+    Checkout(&'a mut Checkout),
+}
+
 #[async_trait]
 pub trait PaymentProcessor {
-    type InitiateArgs: Sync;
     type ProcessArgs: Sync;
     type PrivateData: Sync;
     type PublicData: Sync;
 
-    // this is called synchronously and only once per initiate_payment request
-    async fn initiate_payment(
-        &mut self,
-        _args: &Self::InitiateArgs,
-    ) -> Result<PaymentData<Self::PrivateData, Self::PublicData>, Error> {
-        Ok(PaymentData {
-            correlation_id: None,
-            private_data: None,
-            public_data: None,
-        })
-    }
-
     async fn process_payment(
         &mut self,
         payment: &Payment<Self::PrivateData, Self::PublicData, Self::ProcessArgs>,
@@ -150,11 +146,11 @@ pub enum InternalPaymentState {
 }
 
 // constructor
-async fn new<C: Context + Send>(
+async fn new<'a, C: Context + Send>(
     ctx: &mut C,
-    checkout_id: String,
+    initiator: Initiator<'a>,
     charge_amount: Money,
-    args: <C as PaymentProcessor>::InitiateArgs,
+    maybe_args: Option<<C as PaymentProcessor>::ProcessArgs>,
 ) -> Result<
     Payment<
         <C as PaymentProcessor>::PrivateData,
@@ -166,7 +162,9 @@ async fn new<C: Context + Send>(
     let mut payment = Payment {
         id: Uuid::new_v4().to_string(),
         state: PaymentState::Idle,
-        checkout_id,
+        initiator_id: match &initiator {
+            Initiator::Checkout(co) => InitiatorId::Checkout(co.id.clone()),
+        },
         charge_amount,
         data: PaymentData {
             correlation_id: None,
@@ -175,9 +173,32 @@ async fn new<C: Context + Send>(
         },
     };
 
-    payment.data = ctx.initiate_payment(&args).await?;
     ctx.create_payment(&payment).await?;
 
+    if let Some(args) = maybe_args {
+        let result = ctx.process_payment(&mut payment, &args).await?;
+        payment.data = result.data;
+        match result.state {
+            InternalPaymentState::Incomplete | InternalPaymentState::Failed => {
+                payment.state = PaymentState::Idle;
+            }
+            InternalPaymentState::Succeeded => payment.state = PaymentState::Succeeded,
+            InternalPaymentState::Cancelled => payment.state = PaymentState::Cancelled,
+        }
+
+        ctx.update_payment(&payment).await?;
+
+        match payment.state {
+            PaymentState::Succeeded => match initiator {
+                Initiator::Checkout(co) => co.complete(ctx).await?,
+            },
+            PaymentState::Cancelled => match initiator {
+                Initiator::Checkout(co) => co.revert(ctx).await?,
+            },
+            _ => {}
+        }
+    }
+
     Ok(payment)
 }
 
@@ -219,14 +240,14 @@ pub async fn queue_for_processing<C: Context + Send>(
     }
 }
 
-pub async fn process<C: Context + Send>(
+pub async fn process<'a, C: Context + Send>(
     ctx: &mut C,
     payment: &mut Payment<
         <C as PaymentProcessor>::PrivateData,
         <C as PaymentProcessor>::PublicData,
         <C as PaymentProcessor>::ProcessArgs,
     >,
-    checkout: &mut Checkout,
+    initiator: Initiator<'a>,
 ) -> Result<(), Error> {
     match &payment.state {
         PaymentState::Processing(args) => {
@@ -243,8 +264,12 @@ pub async fn process<C: Context + Send>(
             ctx.update_payment(payment).await?;
 
             match payment.state {
-                PaymentState::Succeeded => checkout.complete(ctx).await,
-                PaymentState::Cancelled => checkout.revert(ctx).await,
+                PaymentState::Succeeded => match initiator {
+                    Initiator::Checkout(co) => co.complete(ctx).await,
+                },
+                PaymentState::Cancelled => match initiator {
+                    Initiator::Checkout(co) => co.revert(ctx).await,
+                },
                 _ => Ok(()),
             }
         }
@@ -252,14 +277,14 @@ pub async fn process<C: Context + Send>(
     }
 }
 
-pub async fn cancel<C: Context + Send>(
+pub async fn cancel<'a, C: Context + Send>(
     ctx: &mut C,
     payment: &mut Payment<
         <C as PaymentProcessor>::PrivateData,
         <C as PaymentProcessor>::PublicData,
         <C as PaymentProcessor>::ProcessArgs,
     >,
-    checkout: &mut Checkout,
+    initiator: Initiator<'a>,
 ) -> Result<(), Error> {
     match &payment.state {
         PaymentState::Cancelled => Ok(()),
@@ -275,7 +300,9 @@ pub async fn cancel<C: Context + Send>(
             payment.state = PaymentState::Cancelled;
             ctx.update_payment(payment).await?;
 
-            checkout.revert(ctx).await
+            match initiator {
+                Initiator::Checkout(co) => co.revert(ctx).await,
+            }
         }
         PaymentState::Processing(_) => Err(new_application_error(
             "PAYMENT_PROCESSING",
